import adani as ad
import oldadani as old
import numpy as np

def test_nll():
    for nll in [True, False]:
        he = ad.HighEnergyCoefficientFunction(3, "2", "g", NLL=nll)
        hehs = ad.HighEnergyHighScaleCoefficientFunction(3, "2", "g", NLL=nll)
        assert he.GetNLL() == nll
        assert hehs.GetNLL() == nll

def test_highenergy_as2_oldadani():
    for kind in ['2', 'L']:
        for channel in ['g', 'q']:
            he = ad.HighEnergyCoefficientFunction(2, kind, channel)
            for x in np.geomspace(1e-5, 1., 100, endpoint=False):
                for m2Q2 in np.geomspace(1e-4, 1e-2, 10):
                    for m2mu2 in np.geomspace(1e-4, 1e-2, 10):
                        res1 = he.fx(x, m2Q2, m2mu2, 0)
                        if (kind == '2' and channel == 'g'):
                            res2 = old.C2_g2_highenergy(x, m2Q2, m2mu2)
                        if (kind == '2' and channel == 'q'):
                            res2 = old.C2_ps2_highenergy(x, m2Q2, m2mu2)
                        if (kind == 'L' and channel == 'g'):
                            res2 = old.CL_g2_highenergy(x, m2Q2, m2mu2)
                        if (kind == 'L' and channel == 'q'):
                            res2 = old.CL_ps2_highenergy(x, m2Q2, m2mu2)
                        np.testing.assert_allclose(res1, res2, rtol=1e-7)

def test_highenergy_as3_oldadani():
    for kind in ['2', 'L']:
        for channel in ['g', 'q']:
            he = ad.HighEnergyCoefficientFunction(3, kind, channel)
            for x in np.geomspace(1e-5, 1., 100, endpoint=False):
                for m2Q2 in np.geomspace(1e-4, 1e-2, 10):
                    for m2mu2 in np.geomspace(1e-4, 1e-2, 10):
                        for nf in range(6 + 1):
                            res1 = he.fx(x, m2Q2, m2mu2, nf)
                            if (kind == '2' and channel == 'g'):
                                res2 = old.C2_g3_highenergy(x, m2Q2, m2mu2, nf, 0)
                            if (kind == '2' and channel == 'q'):
                                res2 = old.C2_ps3_highenergy(x, m2Q2, m2mu2, nf, 0)
                            if (kind == 'L' and channel == 'g'):
                                res2 = old.CL_g3_highenergy(x, m2Q2, m2mu2, nf, 0)
                            if (kind == 'L' and channel == 'q'):
                                res2 = old.CL_ps3_highenergy(x, m2Q2, m2mu2, nf, 0)
                            np.testing.assert_allclose(res1, res2, rtol=1e-7)

def test_highenergyhighscale_as2_oldadani():
    for kind in ['2', 'L']:
        for channel in ['g', 'q']:
            he = ad.HighEnergyHighScaleCoefficientFunction(2, kind, channel)
            for x in np.geomspace(1e-5, 1., 100, endpoint=False):
                for m2Q2 in np.geomspace(1e-4, 1e-2, 10):
                    for m2mu2 in np.geomspace(1e-4, 1e-2, 10):
                        res1 = he.fx(x, m2Q2, m2mu2, 0)
                        if (kind == '2' and channel == 'g'):
                            res2 = old.C2_g2_highenergy_highscale(x, m2Q2, m2mu2)
                        if (kind == '2' and channel == 'q'):
                            res2 = old.C2_ps2_highenergy_highscale(x, m2Q2, m2mu2)
                        if (kind == 'L' and channel == 'g'):
                            res2 = old.CL_g2_highenergy_highscale(x, m2Q2, m2mu2)
                        if (kind == 'L' and channel == 'q'):
                            res2 = old.CL_ps2_highenergy_highscale(x, m2Q2, m2mu2)
                        np.testing.assert_allclose(res1, res2, rtol=1e-7)

def test_highenergyhighscale_as3_oldadani():
    for kind in ['2', 'L']:
        for channel in ['g', 'q']:
            he = ad.HighEnergyHighScaleCoefficientFunction(3, kind, channel)
            for x in np.geomspace(1e-5, 1., 100, endpoint=False):
                for m2Q2 in np.geomspace(1e-4, 1e-2, 10):
                    for m2mu2 in np.geomspace(1e-4, 1e-2, 10):
                        for nf in range(6 + 1):
                            res1 = he.fx(x, m2Q2, m2mu2, nf)
                            if (kind == '2' and channel == 'g'):
                                res2 = old.C2_g3_highenergy_highscale(x, m2Q2, m2mu2, nf, 0)
                            if (kind == '2' and channel == 'q'):
                                res2 = old.C2_ps3_highenergy_highscale(x, m2Q2, m2mu2, nf, 0)
                            if (kind == 'L' and channel == 'g'):
                                res2 = old.CL_g3_highenergy_highscale(x, m2Q2, m2mu2, nf, 0)
                            if (kind == 'L' and channel == 'q'):
                                res2 = old.CL_ps3_highenergy_highscale(x, m2Q2, m2mu2, nf, 0)
                            np.testing.assert_allclose(res1, res2, rtol=1e-7)

# def test_powerterms_as2_oldadani():
#     for kind in ['2', 'L']:
#         for channel in ['g', 'q']:
#             he = ad.PowerTermsCoefficientFunction(2, kind, channel)
#             for x in np.geomspace(1e-5, 1., 100, endpoint=False):
#                 for m2Q2 in np.geomspace(1e-4, 1e-2, 10):
#                     for m2mu2 in np.geomspace(1e-4, 1e-2, 10):
#                         res1 = he.fx(x, m2Q2, m2mu2, 0)
#                         if (kind == '2' and channel == 'g'):
#                             res2 = old.C2_g2_power_terms(x, m2Q2, m2mu2)
#                         if (kind == '2' and channel == 'q'):
#                             res2 = old.C2_ps2_power_terms(x, m2Q2, m2mu2)
#                         if (kind == 'L' and channel == 'g'):
#                             res2 = old.CL_g2_power_terms(x, m2Q2, m2mu2)
#                         if (kind == 'L' and channel == 'q'):
#                             res2 = old.CL_ps2_power_terms(x, m2Q2, m2mu2)
#                         np.testing.assert_allclose(res1, res2, rtol=1e-7)

# def test_powerterms_as3_oldadani():
#     for kind in ['2', 'L']:
#         for channel in ['g', 'q']:
#             he = ad.PowerTermsCoefficientFunction(3, kind, channel)
#             for x in np.geomspace(1e-5, 1., 100, endpoint=False):
#                 for m2Q2 in np.geomspace(1e-4, 1e-2, 10):
#                     for m2mu2 in np.geomspace(1e-4, 1e-2, 10):
#                         for nf in range(6 + 1):
#                             res1 = he.fx(x, m2Q2, m2mu2, nf)
#                             if (kind == '2' and channel == 'g'):
#                                 res2 = old.C2_g3_power_terms(x, m2Q2, m2mu2, nf, 0)
#                             if (kind == '2' and channel == 'q'):
#                                 res2 = old.C2_ps3_power_terms(x, m2Q2, m2mu2, nf, 0)
#                             if (kind == 'L' and channel == 'g'):
#                                 res2 = old.CL_g3_power_terms(x, m2Q2, m2mu2, nf, 0)
#                             if (kind == 'L' and channel == 'q'):
#                                 res2 = old.CL_ps3_power_terms(x, m2Q2, m2mu2, nf, 0)
#                             np.testing.assert_allclose(res1, res2, rtol=1e-7)

# def test_powerterms():
#     for order in range(1, 3 + 1):
#         for kind in ['2', 'L']:
#             for channel in ['g', 'q']:
#                 if channel == 'q' and order == 1:
#                     continue
#                 for NLL in [True, False]:
#                     he = ad.HighEnergyCoefficientFunction(order, kind, channel, NLL)
#                     hehs = ad.HighEnergyHighScaleCoefficientFunction(order, kind, channel, NLL)
#                     pt = ad.PowerTermsCoefficientFunction(order, kind, channel, NLL)
#                     for nf in range(1, 6 + 1):
#                         for m2mu2 in np.geomspace(1e-4, 1e2, 10):
#                             for m2Q2 in np.geomspace(1e-4, 1e2, 10):
#                                 for x in np.geomspace(1e-5, 1., 10, endpoint=False):
#                                     res1 = pt.fxBand(x, m2Q2, m2mu2, nf)

#                                     res2_c = he.fxBand(x, m2Q2, m2mu2, nf).GetCentral() - hehs.fxBand(x, m2Q2, m2mu2, nf).GetCentral()
#                                     tmp1 = he.fxBand(x, m2Q2, m2mu2, nf).GetHigher() - hehs.fxBand(x, m2Q2, m2mu2, nf).GetHigher()
#                                     tmp2 = he.fxBand(x, m2Q2, m2mu2, nf).GetLower() - hehs.fxBand(x, m2Q2, m2mu2, nf).GetLower()
#                                     if tmp1 > tmp2:
#                                         res2_h = tmp1
#                                         res2_l = tmp2
#                                     else:
#                                         res2_h = tmp2
#                                         res2_l = tmp1

#                                     np.testing.assert_allclose(res1.GetCentral(), res2_c, rtol=1e-7)
#                                     np.testing.assert_allclose(res1.GetHigher(), res2_h, rtol=1e-7)
#                                     np.testing.assert_allclose(res1.GetLower(), res2_l, rtol=1e-7)
#                     del he
#                     del pt
#                     del hehs



import adani as ad
import numpy as np

nf = 4
m = 4.92

F2g = ad.ApproximateCoefficientFunction(3, '2', 'g', True, ad.Exact)
FLg = ad.ApproximateCoefficientFunction(3, 'L', 'g', True, ad.Exact)
F2q = ad.ApproximateCoefficientFunction(3, '2', 'q', True, ad.Exact)
FLq = ad.ApproximateCoefficientFunction(3, 'L', 'q', True, ad.Exact)

for x in np.geomspace(1e-4, 1, 5):
    for Q in np.geomspace(10, 100, 5):
        m2Q2 = m**2 / Q**2
        m2mu2 = m2Q2
        print("C2_g^(3)(x=", x, ", Q=", Q, ") = ", F2g.fx(x, m2Q2, m2mu2, nf))
        print("C2_ps^(3)(x=", x, ", Q=", Q, ") = ", F2g.fx(x, m2Q2, m2mu2, nf))
        print("CL_g^(3)(x=", x, ", Q=", Q, ") = ", F2g.fx(x, m2Q2, m2mu2, nf))
        print("CL_ps^(3)(x=", x, ", Q=", Q, ") = ", F2g.fx(x, m2Q2, m2mu2, nf))



from adani import Analytical, DoubleNumerical, MonteCarlo
from adani import Exact, GM, ABMP, KLMV
import adani as ad
import oldadani as old
import numpy as np

def test_as2():
    for kind in ['2', 'L']:
        for channel in ['g', 'q']:
            asy = ad.AsymptoticCoefficientFunction(2, kind, channel)
            asy.SetLegacyPowerTerms(True)
            for m2Q2 in np.geomspace(1e-2, 1e4, 10):
                for m2mu2 in np.geomspace(1e-2, 1e4, 10):
                    for x in np.geomspace(1e-5, 1., 100, endpoint=False):
                        res1 = asy.fx(x, m2Q2, m2mu2, 1)
                        if kind == '2' and channel == 'g':
                            res2 = old.C2_g2_asymptotic(x, m2Q2, m2mu2)
                        if kind == '2' and channel == 'q':
                            res2 = old.C2_ps2_asymptotic(x, m2Q2, m2mu2)
                        if kind == 'L' and channel == 'g':
                            res2 = old.CL_g2_asymptotic(x, m2Q2, m2mu2)
                        if kind == 'L' and channel == 'q':
                            res2 = old.CL_ps2_asymptotic(x, m2Q2, m2mu2)
                        np.testing.assert_allclose(res1, res2, rtol=1e-6)


def test_as3():
    for kind in ['2', 'L']:
        for channel in ['g', 'q']:
            hs_version = Exact if channel == 'q' else ABMP
            asy = ad.AsymptoticCoefficientFunction(3, kind, channel, True, hs_version)
            asy.SetLegacyPowerTerms(True)
            for m2Q2 in np.geomspace(1e-2, 1e4, 10):
                for m2mu2 in np.geomspace(1e-2, 1e4, 10):
                    for nf in range(1, 6+1):
                        for x in np.geomspace(1e-5, 1., 100, endpoint=False):
                            res1 = asy.fx(x, m2Q2, m2mu2, nf)
                            if kind == '2' and channel == 'g':
                                res2 = old.C2_g3_asymptotic(x, m2Q2, m2mu2, nf)
                            if kind == '2' and channel == 'q':
                                res2 = old.C2_ps3_asymptotic(x, m2Q2, m2mu2, nf)
                            if kind == 'L' and channel == 'g':
                                res2 = old.CL_g3_asymptotic(x, m2Q2, m2mu2, nf)
                            if kind == 'L' and channel == 'q':
                                res2 = old.CL_ps3_asymptotic(x, m2Q2, m2mu2, nf)
                            np.testing.assert_allclose(res1, res2, rtol=1e-7)

# def test_asy():
#     for order in range(1, 3 + 1):
#         for kind in ['2', 'L']:
#             for channel in ['g', 'q']:
#                 if order == 1 and channel == 'q':
#                     continue
#                 for hs_version in ["exact", "abmp", "klmv"]:
#                     if channel == 'q' and hs_version == "abmp":
#                         continue
#                     if channel == 'g' and hs_version == "exact":
#                         continue
#                     if order < 3 and hs_version != "exact":
#                         continue
#                     hs = ad.HighScaleCoefficientFunction(order, kind, channel, hs_version)
#                     for nll in [True, False]:
#                         pt = ad.PowerTermsCoefficientFunction(order, kind, channel, nll)
#                         mult = ad.MultiplicativeAsymptotic(order, kind, channel, nll)
#                         asy = ad.AsymptoticCoefficientFunction(order, kind, channel, nll, hs_version)
#                         for nf in range(1, 6 + 1):
#                             for m2mu2 in np.geomspace(1e-4, 1e2, 10):
#                                 for m2Q2 in np.geomspace(1e-4, 1e2, 10):
#                                     for x in np.geomspace(1e-5, 1., 10, endpoint=False):
#                                         # print(order, kind, channel, x, m2Q2, m2mu2, nf, hs_version, nll)
#                                         if kind=='2':
#                                             res1 = pt.fxBand(x, m2Q2, m2mu2, nf) + hs.fxBand(x, m2Q2, m2mu2, nf)
#                                         else:
#                                             res1 = hs.fxBand(x, m2Q2, m2mu2, nf) * mult.fxBand(x, m2Q2, m2mu2, nf)
#                                         res2 = asy.fxBand(x, m2Q2, m2mu2, nf)

#                                         if kind=='2':
#                                             res1_c = pt.fx(x, m2Q2, m2mu2, nf) + hs.fx(x, m2Q2, m2mu2, nf)
#                                         else:
#                                             res1_c = hs.fx(x, m2Q2, m2mu2, nf) * mult.fx(x, m2Q2, m2mu2, nf)
#                                         res2_c = asy.fx(x, m2Q2, m2mu2, nf)

#                                         np.testing.assert_allclose(res1.GetCentral(), res2.GetCentral(), rtol=1e-7)
#                                         np.testing.assert_allclose(res1.GetHigher(), res2.GetHigher(), rtol=1e-7)
#                                         np.testing.assert_allclose(res1.GetLower(), res2.GetLower(), rtol=1e-7)
#                                         np.testing.assert_allclose(res1_c, res1.GetCentral(), rtol=1e-7)
#                                         np.testing.assert_allclose(res2_c, res2.GetCentral(), rtol=1e-7)
#                                         np.testing.assert_allclose(res1_c, res2_c, rtol=1e-7)



import adani as ad
import oldadani as old
import numpy as np

def test_as1():
    for kind in ['2']:
        ml = ad.ThresholdCoefficientFunction(1, kind, 'g')
        ml.SetLegacyThreshold(True)
        for m2Q2 in np.geomspace(1e-2, 1e4, 10):
            xmax = 1. / (1 + 4 * m2Q2)
            for x in np.geomspace(1e-5, xmax, 100, endpoint=False):
                res1 = ml.MuIndependentTerms(x, m2Q2, 1)
                res2 = old.C2_g1_threshold(x, m2Q2)
                np.testing.assert_allclose(res1, res2, rtol=1e-7)

def test_as2():
    for kind in ['2', 'L']:
        thr = ad.ThresholdCoefficientFunction(2, kind, 'g')
        thr.SetLegacyThreshold(True)
        for m2Q2 in np.geomspace(1e-2, 1e4, 10):
            for m2mu2 in np.geomspace(1e-2, 1e4, 10):
                xmax = 1. / (1 + 4 * m2Q2)
                for x in np.geomspace(1e-5, xmax, 100, endpoint=False):
                    res1 = thr.fx(x, m2Q2, m2mu2, 1)
                    if kind == '2':
                        res2 = old.C2_g2_threshold(x, m2Q2, m2mu2)
                    else:
                        res2 = old.CL_g2_threshold(x, m2Q2, m2mu2)
                    np.testing.assert_allclose(res1, res2, rtol=1e-7)

                    res1 = thr.BetaIndependentTerms(x, m2Q2, m2mu2)
                    if kind == '2':
                        res2 = old.C2_g2_threshold_const(x, m2Q2, m2mu2)
                    else:
                        res2 = old.CL_g2_threshold_const(x, m2Q2, m2mu2)
                    np.testing.assert_allclose(res1, res2, rtol=1e-7)

def test_as3():
    for kind in ['2']:
        thr = ad.ThresholdCoefficientFunction(3, kind, 'g')
        thr.SetLegacyThreshold(True)
        for m2Q2 in np.geomspace(1e-2, 1e4, 10):
            for m2mu2 in np.geomspace(1e-2, 1e4, 10):
                xmax = 1. / (1 + 4 * m2Q2)
                for x in np.geomspace(1e-5, xmax, 100, endpoint=False):
                    for nf in range(1, 6 + 1):
                        res1 = thr.fx(x, m2Q2, m2mu2, nf)
                        if kind == '2':
                            res2 = old.C2_g3_threshold(x, m2Q2, m2mu2, nf)
                        else:
                            res2 = old.CL_g3_threshold(x, m2Q2, m2mu2, nf)
                        np.testing.assert_allclose(res1, res2, rtol=1e-7)

                        res1 = thr.BetaIndependentTerms(x, m2Q2, m2mu2)
                        if kind == '2':
                            res2 = old.C2_g3_threshold_const(x, m2Q2, m2mu2)
                        else:
                            res2 = old.CL_g3_threshold_const(x, m2Q2, m2mu2)
                        np.testing.assert_allclose(res1, res2, rtol=1e-7)
